#!/usr/bin/perl

# Copyright (C) 2006 Jim Lynch.
# Contact: jim@fayettedigital.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Oindex.pl - A utility to generate an index using the Xapian/Omega utility,
# scriptindex.  Oindex.pl reads file names from the command line, builds a 
# document suitable for indexing by scriptindex, writes it to a temporary file
# and then executes scriptindex.  
# Use the following command to accomplish this from the scriptindex sample
# find  txt -type f -print |xargs perl Oindex.pl
# This utility is intended to be run from the directory that includes the
# txt directory and the test.index file.
# See the enclosed README.html for additional information
my $URL="/phil/";
use strict;

my $database=$ENV{DB};
if(! -d $database) {
    print "Please set the environment variable, DB, to the path to the database. E. g. export DB='/var/lib/omega/data/default'\n"; 
    print "This needs to be the same location as defined in the omega.conf file.\n";
    exit;
}

#main () 
{
    my $indexDocument="";
    foreach my $file (@ARGV) {
    if( -d $file) { next; }
        open INP,$file;
        my @lines;
        my $dat;
        my $tauthor;
        my $author;
        my $title;
        my $cnt=0;
        # we need the actual file name, to get that we'll locate the last
        # slash character and pull the text following it.
        my $lastSlash=rindex($file,"/");
        my $fn = substr($file,$lastSlash+1);
	# now we want to change it to point to the html file, get the offset to the period
	my $period = index($fn,".");
	$fn = substr($fn,0,$period+1)."html";
        $indexDocument.="url=".$URL.$fn."\n";
        while(<INP>) {
            my $line=$_;
            if($cnt==0) { # First line contains information about this file
                my $string = substr($line,1);
                chomp($string);
                # $string now looks like DATE|AUTHOR|TRANSLATOR|TITLE
                ($dat,$author,$tauthor,$title)=split(/\|/,$string);
                $indexDocument .= "title=$title\n";
                $indexDocument .= "author=$author\n";
                $indexDocument .= "translator=$tauthor\n";
                $indexDocument .= "date=$dat\n";
                $indexDocument .= "body=";
            } else { # this is just part of the body, so add it 
                trim($line);
                if($line) { # don't index blank lines
                    $indexDocument .= "=".$line."\n";
                }
            }
            $cnt++;
        }
        $indexDocument.="\n";
        close INP;
#        Alternatively, we could index each document as it is generated. 
#        To do so, uncomment out the next two lines and comment out the 
#        call to index after the loop.  This will run slower but consume
#        less memory.
#        genindex($indexDocument);
#        $indexDocument="";
    }
    # The indexDocument at this point contains all of the documents, so
    # we'll index it.  
    genindex($indexDocument);
}

sub genindex($)
{
    my $indexDocument=shift;
    # if you change the location/name of this temporary file, be sure to also
    # change it in the scriptindex line
    open OUT,">/tmp/tmp.dmp";
    print OUT $indexDocument;
    # Execute the scripindex program.  If you have placed scriptindex 
    # somewhere else, be sure to adjust the next line.
    my $cmd ="/usr/local/bin/scriptindex $database ./test.index /tmp/tmp.dmp";
    print `$cmd`;
    close OUT;
}

sub trim($)
{
    $_[0] =~ s/^\s+//;
    $_[0] =~ s/\s+$//;
    return;
}
